<?php
/**
 * Kiwiz
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is available through the world-wide-web at the following URI:
 * https://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the PHP License and are unable to
 * obtain it through the web, please send a note to contact@kiwiz.io
 * so we can mail you a copy immediately.
 *
 * @author     Kiwiz <contact@kiwiz.io>
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

class MA_Kiwiz_Model_Api_Client extends MA_Kiwiz_Model_Rest_Client
{
    const XML_PATH_URI = 'kwauth/auth/uri';
    const XML_PATH_USERNAME = 'kwauth/auth/username';
    const XML_PATH_KW_PASSWORD = 'kwauth/auth/password';
    const XML_PATH_KW_SUBSCRIPTION_ID = 'kwauth/auth/subscription_id';
    const XML_PATH_KW_TEST_MODE = 'kwauth/auth/test_mode';

    const XML_PATH_LOG_ENABLE = 'kwgeneral/notification/api_error_log_enable';
    const XML_PATH_NOTIFY_ENABLE = 'kwgeneral/notification/api_error_enable';

    protected $_kwUri;
    protected $_kwTestMode;
    protected $_kwUsername;
    protected $_kwPassword;
    protected $_kwSubscriptionId;
    protected $_kwToken;
    protected $_store;

    protected $_notificationEmails = null;

    public function __construct($store=null)
    {
        $this->_store = (int) $store;
        $this->_kwUri = Mage::getStoreConfig(self::XML_PATH_URI, $this->_store);
        $this->_kwUsername = Mage::getStoreConfig(self::XML_PATH_USERNAME, $this->_store);
        $this->_kwPassword = Mage::getStoreConfig(self::XML_PATH_KW_PASSWORD, $this->_store);
        $this->_kwSubscriptionId = Mage::getStoreConfig(self::XML_PATH_KW_SUBSCRIPTION_ID, $this->_store);

        $this->_kwToken = self::getToken($this->_store);

        parent::__construct(
            $this->_kwUri,
            Mage::getStoreConfig(self::XML_PATH_KW_TEST_MODE, $this->_store),
            'magento',
            'magento:'.Mage::getVersion().'|kiwiz:'.Mage::helper('kiwiz')->getVersion()
        );

        if(!$this->_kwToken) $this->tokenGenerate();
    }



    protected function _getNotificationEmails()
    {
        if($this->_notificationEmails === null) {
            $this->_notificationEmails = Mage::helper('kiwiz/config')->getNotificationEmails();
        }
        return $this->_notificationEmails;
    }

    protected function _log($responseMessage, $data)
    {
        if(Mage::getStoreConfig(self::XML_PATH_LOG_ENABLE))
            Mage::log($responseMessage.PHP_EOL.print_r($data, true), null, 'kiwiz_api.log');

        return $this;
    }

    protected function _notify($responseMessage, $data)
    {
        if(count($this->_getNotificationEmails()) && Mage::getStoreConfig(self::XML_PATH_NOTIFY_ENABLE)) {
            Mage::helper('kiwiz/mail')->adminNotify(
                $this->_getNotificationEmails(),
                Mage::helper('kiwiz')->__('Failed to call Kiwiz API'),
                nl2br($responseMessage.PHP_EOL.print_r($data, true))
            );
        }
        return $this;
    }

    protected function _getResponse(Zend_Http_Response $response, $action)
    {
        $body = $response->getBody();
        if($response->getHeader('Content-type') == 'application/json') {
            $obj = new Varien_Object();
            $obj->addData(Zend_Json::decode($body, Zend_Json::TYPE_ARRAY));
            $body = $obj;
        }

        if($response->getStatus() != 200) {
            $e = new Exception('API KIWIZ ('.$action.') : '.$body->error.' - '.$body->message, $response->getStatus());
            throw $e;
        }
        return $body;
    }

    public function tokenGenerate()
    {
        $action = '/token/generate';
        $data = array(
            'username' => $this->_kwUsername,
            'password' => $this->_kwPassword,
            'subscription_id' => $this->_kwSubscriptionId,
        );
        $response = $this->_getResponse($this->restPost($action, $data), $action);
        $this->_kwToken = $response->getToken();
        self::setToken($this->_kwToken, $this->_store);
    }

    public function __call($method, $args)
    {
        $h = Mage::helper('kiwiz');

        // Retry
        $retry = 0;
        if(isset($args['__kiwiz_retry'])) {
            $retry = 1;
            unset($args['__kiwiz_retry']);
        }

        // Check Arguments
        if(count($args)>1) throw new Exception($h->__('Invalid arguments'));
        $data = null;
        if(isset($args[0])) {
            if (!is_string($args[0]) && !is_array($args[0])) throw new Exception($h->__('Invalid arguments'));
            $data = $args[0];
        }

        // Attachments
        if(isset($data['attachments'])) {
            $attachments = $data['attachments'];
            unset($data['attachments']);
        } else
            $attachments = array();

        // Check method
        $arrMethod = explode('_', strtolower(preg_replace('/(.)([A-Z])/', "$1_$2", $method)));
        if(count($arrMethod)!=3) throw new Exception($h->__('Invalid method'));
        $callType = $arrMethod[0];
        $action = '/'.$arrMethod[1].'/'.$arrMethod[2];
        if(!in_array($callType, array('post', 'get', 'delete', 'put'))) throw new Exception($h->__('Invalid method'));

        // Init http client
        self::getHttpClient()->resetParameters(true);
        $this->setNoReset(true);
        self::getHttpClient()->setHeaders('Authorization', $this->_kwToken);

        // Add attachments
        if(!is_array($attachments)) throw new Exception($h->__('Invalid attachment file'));
        foreach ($attachments as $key => $attachment) {
            if(!($attachment instanceof Varien_Object)) throw new Exception($h->__('Invalid attachment file'));
            self::getHttpClient()->setFileUpload($attachment->getFilename(), $key, $attachment->getFiledata(), $attachment->getCtype());
        }

        // Request
        try {
            $response = $this->_getResponse( $this->{'rest'.ucfirst($callType)}($action, $data) , $action);
        } catch (Exception $e) {
            // Log
            $this->_log($e->getMessage(), $data);

            // If auth require, generate token and retry
            if($retry==0 && $e->getCode() == 401) {
                $this->tokenGenerate();
                $args['__kiwiz_retry'] = 1;
                $response = $this->__call($method, $args);
            } else {
                $this->_notify($e->getMessage(), $data);
                throw $e;
            }
        }

        return $response;
    }

    public static function getToken($store=0)
    {
        $flag = Mage::getModel('kiwiz/flag_token')->setFlagCodeParams($store)->loadSelf();
        return $flag->getFlagData();

    }

    public static function setToken($token, $store=0)
    {
        $flag = Mage::getModel('kiwiz/flag_token')->setFlagCodeParams($store)->loadSelf();
        $flag->setFlagData($token)->save();
    }

    public static function unsetToken($store=0)
    {
        $flag = Mage::getModel('kiwiz/flag_token')->setFlagCodeParams($store)->loadSelf();
        $flag->delete();
    }

    public static function unsetAllTokens()
    {
        MA_Kiwiz_Model_Flag_Token::unsetAll();
    }
}