<?php
/**
 * Kiwiz
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is available through the world-wide-web at the following URI:
 * https://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the PHP License and are unable to
 * obtain it through the web, please send a note to contact@kiwiz.io
 * so we can mail you a copy immediately.
 *
 * @author     Kiwiz <contact@kiwiz.io>
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

abstract class MA_Kiwiz_Model_Document_Abstract
{
    const XML_PATH_MANUFACTURER_ATTRIBUTE_CODE = 'kwgeneral/datasource/manufacturer';
    const XML_PATH_EAN13_ATTRIBUTE_CODE = 'kwgeneral/datasource/ean13';

    protected $_kiwizClient=null;
    protected $_manufacturerAttributeCode=array();
    protected $_ean13AttributeCode=array();

    abstract protected function _getDocumentName();
    abstract protected function _getObjectData($object);

    protected function _getManufacturerAttributeCode($storeId=null)
    {
        if(!isset($this->_manufacturerAttributeCode[(int) $storeId])) {
            $this->_manufacturerAttributeCode[(int) $storeId] = Mage::getStoreConfig(self::XML_PATH_MANUFACTURER_ATTRIBUTE_CODE, $storeId);
        }
        return $this->_manufacturerAttributeCode[(int) $storeId];
    }

    protected function _getEan13AttributeCode($storeId=null)
    {
        if(!isset($this->_ean13AttributeCode[(int) $storeId])) {
            $this->_ean13AttributeCode[(int) $storeId] = Mage::getStoreConfig(self::XML_PATH_EAN13_ATTRIBUTE_CODE, $storeId);
        }
        return $this->_ean13AttributeCode[(int) $storeId];
    }

    protected function _getObjectDataJson($object)
    {
        $data = $this->_getObjectData($object);
        if(!is_array($data)) return null;
        return Zend_Json::encode($data);
    }

    protected function _getKiwizClient($store=null)
    {
        return Mage::getModel('kiwiz/api_client', $store);
    }

    protected function _getObject($object)
    {
        if(!is_object($object))
            $object = Mage::getModel('sales/order_'.$this->_getDocumentName())->load($object);

        if(!$object->getId())
            throw new Exception(Mage::helper('kiwiz')->__('Invalid object ID'));

        return $object;
    }

    protected function _getOrigDocument($object)
    {
        $document = new Varien_Object();

        /** @var Zend_Pdf $pdf */
        $pdf = Mage::getModel('sales/order_pdf_'.$this->_getDocumentName())->getPdf(array($object), true);

        $document->setPdf($pdf);
        $document->setFilename($this->_getDocumentName().'_'.$object->getData('increment_id').'.pdf');
        $document->setFiledata($pdf->render());
        $document->setCtype('application/pdf');

        return $document;
    }

    public function get($object)
    {
        $object = $this->_getObject($object);
        if($object->getData('kiwiz_status') == MA_Kiwiz_Model_Source_Status::NOT_SENT)
            $this->save($object);

        switch ($object->getData('kiwiz_status')) {
            case MA_Kiwiz_Model_Source_Status::DO_NOT_SEND:
                return $this->_getOrigDocument($object);
                break;

            case MA_Kiwiz_Model_Source_Status::SENT:
                if(!$object->getData('kiwiz_block_hash'))
                    throw new Exception(Mage::helper('kiwiz')->__('Invalid block hash'));

                $client = $this->_getKiwizClient($object->getStore()->getId());
                $response = $client->{'get'.ucfirst(strtolower($this->_getDocumentName())).'Get'}(array(
                    'block_hash' => $object->getData('kiwiz_block_hash')
                ));

                $document = new Varien_Object();

                $document->setPdf(MA_Kiwiz_Model_Pdf::parse($response));
                $document->setFilename($object->getIncrementId().'.pdf');
                $document->setFileData($response);
                $document->setCtype('application/pdf');
                return $document;
                break;

            default:
                throw new Exception(Mage::helper('kiwiz')->__('Cannot get the document'));
                break;
        }
    }

    protected function _updateObject($object)
    {
        $resource = Mage::getSingleton('core/resource');
        $writeConnection = $resource->getConnection('core_write');

        $tableName = $writeConnection->quoteTableAs($resource->getTableName('sales_flat_'.$this->_getDocumentName()));
        $tableGridName = $writeConnection->quoteTableAs($resource->getTableName('sales_flat_'.$this->_getDocumentName().'_grid'));
        $objectId = $writeConnection->quote($object->getId());
        $kStatus = $writeConnection->quote($object->getData('kiwiz_status'));
        $kMessage = $object->getData('kiwiz_message') ? $writeConnection->quote($object->getData('kiwiz_message')) : "NULL";
        $kBlockH = $object->getData('kiwiz_block_hash') ? $writeConnection->quote($object->getData('kiwiz_block_hash')) : "NULL";
        $kFileH = $object->getData('kiwiz_file_hash') ? $writeConnection->quote($object->getData('kiwiz_file_hash')) : "NULL";

        $writeConnection->query("
            UPDATE $tableName SET
              `kiwiz_status` = $kStatus,
              `kiwiz_message` = $kMessage,
              `kiwiz_block_hash` = $kBlockH,
              `kiwiz_file_hash` = $kFileH
            WHERE `entity_id` = $objectId
        ");

        $writeConnection->query("
            UPDATE $tableGridName SET
              `kiwiz_status` = $kStatus,
              `kiwiz_message` = $kMessage,
              `kiwiz_block_hash` = $kBlockH,
              `kiwiz_file_hash` = $kFileH
            WHERE `entity_id` = $objectId
        ");
    }

    public function save($object)
    {
        $object = $this->_getObject($object);
        if($object->getData('kiwiz_status') != MA_Kiwiz_Model_Source_Status::NOT_SENT) return;

        try {
            $client = $this->_getKiwizClient($object->getStore()->getId());
            $response = $client->{'post'.ucfirst(strtolower($this->_getDocumentName())).'Save'}(array(
                'attachments' => array(
                    'document' => $this->_getOrigDocument($object)
                ),
                'data' => $this->_getObjectDataJson($object)
            ));

            $object->addData(array(
                'kiwiz_status' => MA_Kiwiz_Model_Source_Status::SENT,
                'kiwiz_message' => null,
                'kiwiz_block_hash' => $response->getBlockHash(),
                'kiwiz_file_hash' => $response->getFileHash()
            ));
        } catch(Exception $e) {
            $object->addData(array(
                'kiwiz_status' => MA_Kiwiz_Model_Source_Status::NOT_SENT,
                'kiwiz_message' => $e->getMessage(),
                'kiwiz_block_hash' => null,
                'kiwiz_file_hash' => null
            ));
        }

        $this->_updateObject($object);
    }

    public function getPdf($objects = array(), $forceOriginal=false)
    {
        if($forceOriginal)
            return Mage::getModel('sales/order_pdf_'.$this->_getDocumentName())->getPdf($objects, true);

        if($objects instanceof Varien_Data_Collection)
            $objects = $objects->getItems();

        if(!is_array($objects))
            $objects = array($objects);

        if(count($objects) == 1) {
            $pdf = $this->get(reset($objects))->getPdf();
        } else {
            $pdf = new Zend_Pdf();
            foreach ($objects as $object) {
                $pages = $this->get($object)->getPdf();
                foreach ($pages->pages as $page) {
                    $template = clone $page;
                    $pdfPage = new Zend_Pdf_Page($template);
                    $pdf->pages[] = $pdfPage;
                }
            }
        }
        return $pdf;
    }
}